/* 
 *
 * Created on 9. Dezember 2010, 12:58
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef ALEXELEMENTS_H
#define	ALEXELEMENTS_H

#include <map>
#include <vector>
#include <algorithm>
#include "AlexTypes.h"
#include "AlexException.h"


/*! \brief AlexElement is the base class for all finite elements like
 *         trias, quads etc.
 *
 * An AlexElement provides
 * the functionality common to all elements. This is the property id,
 * visibility status and locked status. If locked, creation of new elements
 * causes an exception. This avoids creation of elements by access via non
 * existing ids.
 */

class AlexElement {
public:
    AlexElement(AlexId id, AlexId myPropertyId, AlexId myPartId){
        this->visible=false;
        this->id = id;
        this->propertyId=myPropertyId;
        this->partId=myPartId;
    };
    AlexElement(){
        this->visible=false;
        this->propertyId=ALEX_INVALID_ID;
    };
    ~AlexElement(){};
    AlexId getId(){ return this->id;}
    void setId(AlexId newId){ this->id=newId;}
    AlexId getPropertyId(){ return this->propertyId;}
    void setPropertyId(AlexId newProperty){ this->propertyId=newProperty;}
    AlexId getPartId(){ return this->partId;}
    void setPartId(AlexId newPart){ this->partId=newPart;}
    bool isVisible(){return this->visible;}
    void setVisibility(bool isVisible){this->visible=isVisible;}
    void toggleVisibility(){this->visible = !this->visible;}
    void lock() {}
    void unlock() {;}
    virtual bool containsNode(AlexId id)=0;
private:
    AlexId id;
    AlexId propertyId;
    AlexId partId;
    bool visible;
};

template <class MyElement>
class AlexElements {
public:
    std::map<AlexId, MyElement *> map;
    typedef typename std::map<AlexId, MyElement>::iterator ElementIterator;
    MyElement *  operator[](const AlexId id){
        if(this->map.find(id)==this->map.end()) throw(ALEX_INVALID_ID_EXCEPTION);
        return this->map[id];};

    virtual void insertElement(MyElement *E)=0;
    std::vector<AlexId> getElementsWithPart(AlexId id){
        std::vector<AlexId> result;
        for(ElementIterator i=map.begin(); i!=map.end(); i++){
            if(i->second.getPartId() == id)
                result.push_back(i->first);
        }
        return result;
    }
    std::vector<AlexId> getElementsWithNode(AlexId id){
        std::vector<AlexId> result;
        for(ElementIterator i=map.begin(); i!=map.end(); i++){
            if(i->second.containsNode(id))
                result.push_back(i->first);
        }
        return result;
    }
    typename std::map<AlexId, MyElement>::iterator begin() {return map.begin();}
    typename std::map<AlexId, MyElement>::iterator end() {return map.end();}

};

class AlexNode {
public:
    AlexId id;
    AlexTriple pos;
    AlexNode(AlexId id, AlexReal x, AlexReal y, AlexReal z){
        this->id = id;
        pos.x=x; pos.y=y; pos.z=z;
    }
};

class AlexNodes {
public:
    std::vector<AlexNode> Nodes;
    std::vector<AlexNode>::iterator begin(){return Nodes.begin();}
    std::vector<AlexNode>::iterator end(){return Nodes.end();}
    AlexNode& operator[](const AlexId id){return this->Nodes[getNodeIndexById(id)];}
    void insert(AlexNode n){
        Nodes.push_back(n);
    }
    AlexIndex getNodeIndexById(AlexId myId){
        AlexIndex lo = 0;
        AlexIndex hi = Nodes.size();
        AlexIndex search;
        for(search= (lo+hi)/2; search= (lo+hi)/2; Nodes[search].id!=myId){
            if(Nodes[search].id>myId){
                hi=search;
            } else {
                lo=search;
            }
            if(hi-lo<3){
                for(search=lo; search<=hi; search++){
                    if(Nodes[search].id==myId) return search;
                }
                throw(ALEX_INVALID_ID_EXCEPTION);
            }
        }
        return search;
    };
    AlexNode& getNodeById(AlexId myId){
        int myIndex = getNodeIndexById(myId);
        if(myIndex >= 0) return Nodes[myIndex];
        throw(ALEX_INVALID_ID_EXCEPTION);
    }
   static bool cmpNodes(const AlexNode n1, const AlexNode n2){
        return n1.id < n2.id;
    }
    void sortNodes(){
        std::sort(Nodes.begin(), Nodes.end(), cmpNodes);
    }
};

typedef std::vector<AlexNode>::iterator NodeIterator;

#endif	/* ALEXELEMENTS_H */

