!
!> \file ModElements.f95
!! \brief Element Modul
!
!> Modul: ModElements
!> \author Klaus Ramstöck klra67@freenet.de
!> \date 26.03.2011
! (C) 2011 Klaus Ramstöck klra67@freenet.de
!
! This program is free software; you can redistribute it and/or modify it under the terms
!  of the GNU General Public License as published by the Free Software Foundation; either
!  version 3 of the License, or (at your option) any later version.
!
! This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
! without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
! See the GNU General Public License for more details.
!
! You should have received a copy of the GNU General Public License along with this program;
!  if not, see <http://www.gnu.org/licenses/>.
!
MODULE ModElements
    USE AlexTypes
    USE ListOps
    IMPLICIT NONE
    PRIVATE

    PUBLIC :: Element, ElementP, Elements
    PUBLIC :: initializeElements, insertElement, getElement
    PUBLIC :: appendElement, sortElements

    !> Base Type for all Elements
    TYPE Element
        INTEGER :: EType
        INTEGER(kind = AlexIdKind) :: Id
        INTEGER(kind = AlexIdKind) :: PropertyId
        INTEGER(kind = AlexIdKind) :: PartId
        INTEGER(kind = AlexIdKind), POINTER :: NodeIds(:)
    END TYPE Element

    !> Element pointer type, used for referers
    TYPE ElementP
        TYPE(Element), POINTER :: ep
    END TYPE ElementP

    !> Element list
    TYPE Elements
        INTEGER(AlexIdKind) :: fill
        TYPE(Element), POINTER :: e(:)
    END TYPE Elements

    TYPE(Elements), POINTER :: getterGlobal

    !> Interface to insert different Elements
    INTERFACE insertElement
        MODULE PROCEDURE insertElementE
        module PROCEDURE insertElementN
        module PROCEDURE insertElementL
    END INTERFACE insertElement

    !> Interface to append different Elements
    INTERFACE appendElement
        MODULE PROCEDURE appendElementE
        module PROCEDURE appendElementN
        module PROCEDURE appendElementL
    END INTERFACE appendElement

CONTAINS

    !> This must be called once on each new elements list,
    !> before anything else is done
    SUBROUTINE initializeElements(this, osz, stat)
        TYPE(Elements), INTENT(inout) :: this
        INTEGER(AlexIdKind), INTENT(in), OPTIONAL :: osz
        INTEGER, INTENT(out), OPTIONAL :: stat
        INTEGER :: sz
        sz = DefaultTableSize
        IF (PRESENT(osz)) sz = osz
        this % fill = 0
        if (present(stat)) then
            ALLOCATE(this % e(sz), stat = stat)
        else
            ALLOCATE(this % e(sz))
        end if
    END SUBROUTINE initializeElements

    !> This must be called after the last append, befor
    !> getElement works, if append is used instead insert
    SUBROUTINE sortElements(this)
        TYPE(Elements), INTENT(inout), target :: this
        getterGlobal => this
        call ListSort(this % fill, getter, swap)
    END SUBROUTINE sortElements


    !> Insert one element into the list of Elements
    SUBROUTINE insertElementE(this, e, stat)
        TYPE(Elements), INTENT(inout) :: this
        TYPE(Element), INTENT(in) :: e
        INTEGER, INTENT(out), OPTIONAL :: stat
        INTEGER :: i
        IF (this % fill .eq. size(this % e)) call grow(this, stat)
        DO i = 1, this % fill
            IF (this % e(i) % id .gt. e % id) EXIT
        END DO
        IF (i .le. this % fill) call shift(this, i)
        this % e(i) = e
        this % fill = this % fill + 1
    END SUBROUTINE insertElementE

    !> Append one element into the list of Elements, destroying the sort order
    SUBROUTINE appendElementE(this, e, stat)
        TYPE(Elements), INTENT(inout) :: this
        TYPE(Element), INTENT(in) :: e
        INTEGER, INTENT(out), OPTIONAL :: stat
        INTEGER :: i
        IF (this % fill .eq. size(this % e)) call grow(this, stat)
        this % fill = this % fill + 1
        this % e(this % fill) = e
    END SUBROUTINE appendElementE


    !> Insert one element with up to 8 nodes into the list of elements
    SUBROUTINE insertElementn(this, etype, id, partId, propertyId, n1, n2, n3, n4, n5, n6, n7, n8, stat)
        TYPE(Elements), INTENT(inout) :: this
        integer, intent(in) :: etype, id, partId, propertyId
        integer(kind = AlexIdKind), intent(in), optional :: n1, n2, n3, n4, n5, n6, n7, n8
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(ELEMENT) :: e
        integer :: sz = 0, i
        if (present(n1)) sz = sz + 1
        if (present(n2)) sz = sz + 1
        if (present(n3)) sz = sz + 1
        if (present(n4)) sz = sz + 1
        if (present(n5)) sz = sz + 1
        if (present(n6)) sz = sz + 1
        if (present(n7)) sz = sz + 1
        if (present(n8)) sz = sz + 1
        if (present(stat)) then
            ALLOCATE(e % nodeIds(sz), stat = stat)
        else
            ALLOCATE(e % nodeIds(sz))
        end if
        e % eType = eType
        e % id = id
        e % partId = partId
        e % propertyId = propertyId
        if (present(n1)) e % nodeIds(1) = n1
        if (present(n2)) e % nodeIds(2) = n2
        if (present(n3)) e % nodeIds(3) = n3
        if (present(n4)) e % nodeIds(4) = n4
        if (present(n5)) e % nodeIds(5) = n5
        if (present(n6)) e % nodeIds(6) = n6
        if (present(n7)) e % nodeIds(7) = n7
        if (present(n8)) e % nodeIds(8) = n8
        call insertElement(this, e, stat)
    END SUBROUTINE

    ! > Insert one element with up to 8 nodes into the list of elements
    SUBROUTINE appendElementn(this, etype, id, partId, propertyId, n1, n2, n3, n4, n5, n6, n7, n8, stat)
        TYPE(Elements), INTENT(inout) :: this
        integer, intent(in) :: etype
        integer(AlexIdKind), intent(in) :: id, partId, propertyId
        integer(kind = AlexIdKind), intent(in), optional :: n1, n2, n3, n4, n5, n6, n7, n8
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(ELEMENT) :: e
        integer :: sz = 0, i
        if (present(n1)) sz = sz + 1
        if (present(n2)) sz = sz + 1
        if (present(n3)) sz = sz + 1
        if (present(n4)) sz = sz + 1
        if (present(n5)) sz = sz + 1
        if (present(n6)) sz = sz + 1
        if (present(n7)) sz = sz + 1
        if (present(n8)) sz = sz + 1
        if (present(stat)) then
            ALLOCATE(e % nodeIds(sz), stat = stat)
        else
            ALLOCATE(e % nodeIds(sz))
        end if
        e % eType = eType
        e % id = id
        e % partId = partId
        e % propertyId = propertyId
        if (present(n1)) e % nodeIds(1) = n1
        if (present(n2)) e % nodeIds(2) = n2
        if (present(n3)) e % nodeIds(3) = n3
        if (present(n4)) e % nodeIds(4) = n4
        if (present(n5)) e % nodeIds(5) = n5
        if (present(n6)) e % nodeIds(6) = n6
        if (present(n7)) e % nodeIds(7) = n7
        if (present(n8)) e % nodeIds(8) = n8
        call appendElement(this, e, stat)
    END SUBROUTINE

    !> Insert one Element given a list of nodes
    SUBROUTINE insertElementL(this, etype, id, partId, propertyId, nIds, stat)
        TYPE(Elements), INTENT(inout) :: this
        integer, intent(in) :: etype, id, partId, propertyId
        integer(kind = AlexIdKind), intent(in), target :: nIds(:)
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(ELEMENT) :: e
        e % eType = eType
        e % id = id
        e % partId = partId
        e % propertyId = propertyId
        e % nodeIds => nIds
        call insertElement(this, e, stat)
    END SUBROUTINE

    !> Insert one Element given a list of nodes
    SUBROUTINE appendElementL(this, etype, id, partId, propertyId, nIds, stat)
        TYPE(Elements), INTENT(inout) :: this
        integer, intent(in) :: etype, id, partId, propertyId
        integer(kind = AlexIdKind), intent(in), target :: nIds(:)
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(ELEMENT) :: e
        e % eType = eType
        e % id = id
        e % partId = partId
        e % propertyId = propertyId
        e % nodeIds => nIds
        call appendElement(this, e, stat)
    END SUBROUTINE

    !> Get one element from the list
    FUNCTION getElement(this, id, stat) RESULT(e)
        TYPE(Elements), INTENT(in), target :: this
        INTEGER(AlexIdKind), INTENT(in) :: id
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(Element) :: e
        INTEGER(AlexIdKind) :: i
        getterGlobal => this
        i = ListFind(id, this % fill, getter)
        if (i .eq. invalidId) go to 8000
        e = this % e(i)
        goto 9000
        8000 continue
        if (present(stat)) then
            stat = ENotFound
        ELSE
            print *, "Element ", id, " not found in ", this % fill, " Elements of type ", this % e(i) % etype
            stop
        end if
        9000 continue
    END FUNCTION getElement

    SUBROUTINE grow(this, stat)
        TYPE(Elements), INTENT(inout) :: this
        INTEGER, INTENT(out), OPTIONAL :: stat
        TYPE(Element), POINTER :: enew(:)
        INTEGER :: sz, sznew, i
        sz = size(this % e)
        sznew = sz * 2
        if (present(stat)) then
            ALLOCATE(enew(sznew), stat = stat)
        else
            ALLOCATE(enew(sznew))
        end if
        do i = 1, this % fill
            enew(i) = this % e(i)
        END DO
        if (present(stat)) then
            DEALLOCATE(this % e, stat = stat)
        else
            DEALLOCATE(this % e)
        end if
        this % e => enew
    END SUBROUTINE grow

    SUBROUTINE shift(this, i)
        TYPE(Elements), INTENT(inout) :: this
        INTEGER, INTENT(in), OPTIONAL :: i
        INTEGER :: j
        do j = this % fill, i, -1
            this % e(j + 1) = this % e(j)
        end do
    END SUBROUTINE shift

    INTEGER(AlexIdKind) FUNCTION getter(i)
        INTEGER(AlexIdKind), intent(in) :: i
        getter = getterGlobal % e(i) % id
    END FUNCTION getter

    SUBROUTINE swap(i, j)
        INTEGER(AlexIdKind), intent(in) :: i, j
        TYPE(Element) :: tmp
        tmp = getterGlobal % e(i)
        getterGlobal % e(i) = getterGlobal % e(j)
        getterGlobal % e(j) = tmp
    END SUBROUTINE swap
END MODULE ModElements
