/*
 * Element.java - base class for all FEM Elements like quads, trias etc.
 *
 * Created on 9. Dezember 2010, 12:58
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */


package Alex.Elements;

/**
 * Base class for all elements. Elements have a property id, specifiying where to pull
 * material properties. They have a part id, specifying which part they belong to.
 * They are either visible or invisible. Finally, they have an array of nodes which
 * make up the element.
 * @author Klaus Ramstöck
 * @version 0.1
 */
public class Element {
    private    long id;
    private    long propertyId;
    private    long partId;
    private    boolean visible;
    private    long nodes[];

    /**
     * You should not call this constructor directly. There
     * should be no instances of Elements, so maybe this should be
     * abstract.
     * @param myPropertyId  id of the property
     * @param myPartId  id of the part
     * @param myNodes[]  array of nodes making up the element
     */
    Element(long myId, long myPropertyId, long myPartId, long myNodes[]){
        id = myId;
        propertyId = myPropertyId;
        partId = myPartId;
        nodes=myNodes;
        visible = true;
    };

     /**
     *
     * @return  id of this element
     */
    public long getId() {return id;}

    /**
     *
     * @param id - set the id of this element
     */
    public void setId(long id) {this.id=id;}
   /**
     *
     * @return property id of this element
     */
    public long getPropertyId() {return propertyId;}

    /**
     *
     * @param id - set the property id of this element
     */
    public void setPropertyId(long id) {propertyId=id;}
    /**
     *
     * @return part id of this element
     */
    public long getPartId() {return partId;}
    /**
     *
     * @param id - set part id of this element
     */
    public void setPartId(long id) {partId=id;}
    /**
     *
     * @return visibility of this element
     */
    public boolean getVisibility() {return visible;}
    /**
     *
     * @param v set visibility of this element to v
     */
    public void setVisibility(boolean v) {visible=v;}
    /**
     * negate visibility of this element
     */
    public void toggleVisibility() { visible = !visible;}
    /**
     *
     * @return array of node ids
     */
    public long[] getNodes() { return nodes;}
    /**
     *
     * @return node 1. Does not check node exists.
     */
    public long n1() {return nodes[0];}
    /**
     *
     * @return node 2. Does not check node exists.
     */
    public long n2() {return nodes[1];}
    /**
     *
     * @return node 3. Does not check node exists.
     */
    public long n3() {return nodes[2];}
    /**
     *
     * @return node 4. Does not check node exists.
     */
    public long n4() {return nodes[3];}
    /**
     *
     * @return node 5. Does not check node exists.
     */
    public long n5() {return nodes[4];}
    /**
     *
     * @return node 6. Does not check node exists.
     */
    public long n6() {return nodes[5];}
    /**
     *
     * @return node 7. Does not check node exists.
     */
    public long n7() {return nodes[6];}
    /**
     *
     * @return node 8. Does not check node exists.
     */
    public long n8() {return nodes[7];}

    public boolean isReferer(long nodeId){
        for(long nid: nodes){
            if(nid==nodeId) return true;
        }
        return false;
    }
}
