/*
 * Nodes.java - FEM Geomerty information
 *
 * Klaus Ramstöck, 2011-03-12 klra67@freenet.de
 *
 * Copyright (C) 2010 Klaus Ramstöck klra67@freenet.de
 *
 * This program is free software; you can redistribute it and/or modify it under the terms
 *  of the GNU General Public License as published by the Free Software Foundation; either
 *  version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with this program;
 *  if not, see <http://www.gnu.org/licenses/>.
 *
 */

package Alex.Elements;
import java.util.ArrayList;

/**
 *
 * @author Klaus Ramstöck klra67@freenet.de
 */
public class Node {
    private Vector p;
    private long id;
    private ArrayList<Element> referers;

    /**
     * Construct a node by id and x/y/z position
     * @param myId
     * @param xx
     * @param yy
     * @param zz
     */
   public  Node(long myId, float xx, float yy, float zz){
        id=myId;
        p = new Vector(xx,yy,zz);
        referers = new ArrayList<Element>();
    }

    /**
     * construct a Node by id and vector v
     * @param myId
     * @param v
     */
    public Node(long myId, Vector v){
        id=myId;
        p = v;
        referers = new ArrayList<Element>();
    }

    /**
     * Get node id
     * @return id
     */
    public long getId(){return id;}

    /**
     * set node id
     * @param myId
     */
    public void setId(long myId){id=myId;}

    /**
     * get node position
     * @return Vector p
     */
    public Vector getPoint() {return p;}

    /**
     * set node position
     * @param Vector v
     */
    public void setPoint(Vector v){p=v;}

    /**
     * get node x position
     * @return float X
     */
    public float X(){return p.x;}

    /**
     * get node y position
     * @return float Y
     */
    public float Y(){return p.y;}

    /**
     * get node z position
     * @return float Z
     */
    public float Z(){return p.z;}

    /**
     * set node X position
     * @param float xx
     */
    public void setX(float xx){p.x=xx;}

    /**
     * set node y position
     * @param float yy
     */
    public void setY(float yy){p.y=yy;}

    /**
     * set node z position
     * @param float zz
     */
    public void setZ(float zz){p.z=zz;}

    /**
     * get distance to node n
     * @param Node n
     * @return double dist
     */
    public double dist(Node n){
        return p.dist(n.getPoint());
    }

    /**
     * get distance Vector to Node n
     * @param Node n
     * @return Vector d
     */
    public Vector distV(Node n){
        Vector d = new Vector(n.getPoint());
        return d.sub(p);
    }

    /**
     * Add element to referers
     * @param e element to be added
     */
    public void addReferer(Element e){
        referers.add(e);
    }

    /**
     * remove element from referers
     * @param e element to be removed
     */
    public void removeReferer(Element e){
        referers.remove(e);
    }

    /**
     * Get ArrayList of referers
     * @return referers
     */
    public ArrayList<Element> getReferers(){
        return referers;
    }

    /**
     * return true if Element e refers to this node
     * @param e
     * @return
     */
    public boolean isReferer(Element e){
        for(Element x: referers){
            if(e==x) return true;
        }
        return false;
    }
}
